<?php
/**
 * Custom functions that act independently of the theme templates
 *
 * Eventually, some of the functionality here could be replaced by core features
 *
 * @package commercegurus
 */

/**
 * Get our wp_nav_menu() fallback, wp_page_menu(), to show a home link.
 */
function economistcommercegurus_page_menu_args( $args ) {
	$args['show_home'] = true;
	return $args;
}

add_filter( 'wp_page_menu_args', 'economistcommercegurus_page_menu_args' );

/**
 * Adds custom classes to the array of body classes.
 */
function economistcommercegurus_body_classes( $classes ) {

	$economistcommercegurus_sticky_menu_class	 = '';
	$economistcommercegurus_show_page_title		 = '';
	$economistcommercegurus_header_bg_style		 = '';

	$economistcommercegurus_sticky_menu_class = economistcommercegurus_getoption( 'economistcommercegurus_sticky_menu' );

	// Adds a class of group-blog to blogs with more than 1 published author
	if ( is_multi_author() ) {
		$classes[] = 'group-blog';
	}

	// Adds a class if the Minimal Header Style is selected
	if ( function_exists( 'get_field' ) ) {
		$economistcommercegurus_show_page_title = get_field( 'show_page_title' );
	}

	if ( $economistcommercegurus_show_page_title == 'Minimal' ) {
		$classes[] = 'cg-minimal';
	}

	$classes[] = 'cg-sticky-enabled';

	return $classes;
}

add_filter( 'body_class', 'economistcommercegurus_body_classes' );

/**
 * Filter in a link to a content ID attribute for the next/previous image links on image attachment pages
 */
function economistcommercegurus_enhanced_image_navigation( $url, $id ) {
	if ( !is_attachment() && !wp_attachment_is_image( $id ) )
		return $url;

	$image = get_post( $id );
	if ( !empty( $image->post_parent ) && $image->post_parent != $id )
		$url .= '#main';

	return $url;
}

add_filter( 'attachment_link', 'economistcommercegurus_enhanced_image_navigation', 10, 2 );

/**
 * Filters wp_title to print a neat <title> tag based on what is being viewed.
 */
function economistcommercegurus_wp_title( $title, $sep ) {
	global $page, $paged;

	if ( is_feed() )
	//	return $title;
	// Add the blog name
		$title .= get_bloginfo( 'name' );

	// Add a page number if necessary:
	if ( $paged >= 2 || $page >= 2 )
		$title .= " $sep " . sprintf( esc_html__( 'Page %s', 'economist' ), max( $paged, $page ) );

	return $title;
}

add_filter( 'wp_title', 'economistcommercegurus_wp_title', 10, 2 );

function economistcommercegurus_add_menu_parent( $items ) {
	$parents = array();
	foreach ( $items as $item ) {
		if ( $item->menu_item_parent && $item->menu_item_parent > 0 ) {
			$parents[] = $item->menu_item_parent;
		}
	}
	return $items;
}

add_filter( 'wp_nav_menu_objects', 'economistcommercegurus_add_menu_parent' );


/* Boxed class */
if ( !function_exists( 'economistcommercegurus_boxed_class' ) ) {

	function economistcommercegurus_boxed_class( $classes ) {

		$economistcommercegurus_boxed_status = '';
		$economistcommercegurus_boxed_status = economistcommercegurus_getoption( 'container_style' );


		if ( !empty( $_SESSION['economistcommercegurus_boxed'] ) ) {
			$economistcommercegurus_boxed_status = esc_html( $_SESSION['economistcommercegurus_boxed'] );
		}

		if ( ( $economistcommercegurus_boxed_status == 'yes' ) || ( $economistcommercegurus_boxed_status == 'boxed' ) ) :
			$classes[] = 'boxed';
		else:
			$classes[] = "";
		endif;

		return $classes;
	}

}

add_filter( 'body_class', 'economistcommercegurus_boxed_class' );

// Initialize some global js vars

if ( !function_exists( 'economistcommercegurus_js_init' ) ) {

	function economistcommercegurus_js_init() {
		?>
		<script type="text/javascript">
			var view_mode_default = '<?php echo esc_js( $economistcommercegurus_options['product_layout'] ) ?>';
			var cg_sticky_default = '<?php echo esc_js( $economistcommercegurus_options['economistcommercegurus_sticky_menu'] ) ?>';
		</script>
		<?php
	}

}

// Util function for building VC row styles - replaces default VC buildstyle function

if ( !function_exists( 'economistcommercegurus_build_style' ) ) {

	function economistcommercegurus_build_style( $bg_image = '', $bg_color = '', $bg_image_repeat = '', $font_color = '',
											  $padding = '', $padding_bottom = '', $padding_top = '', $margin_bottom = '' ) {
		$has_image	 = false;
		$style		 = '';
		if ( (int) $bg_image > 0 && ( $image_url	 = wp_get_attachment_url( $bg_image, 'large' ) ) !== false ) {
			$has_image = true;
			$style .= "background-image: url(" . $image_url . ");";
		}
		if ( !empty( $bg_color ) ) {
			$style .= 'background-color: ' . $bg_color . ';';
		}
		if ( !empty( $bg_image_repeat ) && $has_image ) {
			if ( $bg_image_repeat === 'cover' ) {
				$style .= "background-repeat:no-repeat;background-size: cover;";
			} elseif ( $bg_image_repeat === 'contain' ) {
				$style .= "background-repeat:no-repeat;background-size: contain;";
			} elseif ( $bg_image_repeat === 'no-repeat' ) {
				$style .= 'background-repeat: no-repeat;';
			}
		}
		if ( !empty( $font_color ) ) {
			$style .= 'color: ' . $font_color . ';';
		}
		if ( $padding != '' ) {
			$style .= 'padding: ' . ( preg_match( '/(px|em|\%|pt|cm)$/', $padding ) ? $padding : $padding . 'px' ) . ';';
		}
		if ( $padding_bottom != '' ) {
			$style .= 'padding-bottom: ' . ( preg_match( '/(px|em|\%|pt|cm)$/', $padding_bottom ) ? $padding_bottom : $padding_bottom . 'px' ) . ';';
		}
		if ( $padding_top != '' ) {
			$style .= 'padding-top: ' . ( preg_match( '/(px|em|\%|pt|cm)$/', $padding_top ) ? $padding_top : $padding_top . 'px' ) . ';';
		}
		if ( $margin_bottom != '' ) {
			$style .= 'margin-bottom: ' . ( preg_match( '/(px|em|\%|pt|cm)$/', $margin_bottom ) ? $margin_bottom : $margin_bottom . 'px' ) . ';';
		}
		return empty( $style ) ? $style : ' style="' . $style . '"';
	}

}

// Hi ThemeForest review team! This is a safe filter for cleaning up CommerceGurus shortcode output only!
// Credits to bitfade for this solution - https://gist.github.com/bitfade/4555047
// Ref - http://themeforest.net/forums/thread/how-to-add-shortcodes-in-wp-themes-without-being-rejected/98804?page=4#996848

add_filter( "the_content", "economistcommercegurus_content_filter" );

function economistcommercegurus_content_filter( $content ) {

	// array of custom shortcodes requiring the fix
	$block = join( "|", array( "economistcommercegurus_content_strip", "vc_button", "cg_video_banner" ) );

	// opening tag
	$rep = preg_replace( "/(<p>)?\[($block)(\s[^\]]+)?\](<\/p>|<br \/>)?/", "[$2$3]", $content );

	// closing tag
	$rep = preg_replace( "/(<p>)?\[\/($block)](<\/p>|<br \/>)?/", "[/$2]", $rep );

	return $rep;
}

add_action( 'economistcommercegurus_page_title', 'economistcommercegurus_page_title_callback' );

function economistcommercegurus_page_title_callback() {
	global $post, $economistcommercegurus_options;
	$economistcommercegurus_show_page_title				 = '';
	$economistcommercegurus_show_below_content_features	 = '';
	$economistcommercegurus_share_options				 = '';
	$economistcommercegurus_header_image				 = '';
	$economistcommercegurus_header_image_style			 = '';
	$economistcommercegurus_cta_heading					 = '';

	$economistcommercegurus_share_options = economistcommercegurus_getoption( 'economistcommercegurus_share_options' );

	if ( function_exists( 'get_field' ) ) {
		$economistcommercegurus_show_page_title				 = get_field( 'show_page_title' );
		$economistcommercegurus_show_below_content_features	 = get_field( 'show_below_content_features' );
	}
	?>

	<?php
	$economistcommercegurus_cta_heading = economistcommercegurus_getoption( 'economistcommercegurus_cta_heading' );
	?>

	<?php if ( ( $economistcommercegurus_show_page_title !== 'Hide' ) && ( $economistcommercegurus_show_page_title !== 'Minimal' ) ) { ?>

		<?php
		if ( has_post_thumbnail( $post->ID ) ):
			$economistcommercegurus_header_image = wp_get_attachment_image_src( get_post_thumbnail_id( $post->ID ), 'full' );
		endif;

		if ( $economistcommercegurus_header_image ) {
			$economistcommercegurus_header_image_style = 'background-image: url(' . $economistcommercegurus_header_image[0] . '); ';
		}
		?>


		<div class="header-wrapper">
			<div class="cg-hero-bg"></div>
			<div class="overlay"></div> 

			<div class="container">
				<div class="row">
					<div class="col-xs-12 col-lg-12 col-md-12">
						<header class="entry-header">
							<?php if ( function_exists( 'yoast_breadcrumb' ) ) { ?>
								<?php yoast_breadcrumb( '<p class="sub-title">', '</p>' ); ?>					
							<?php } ?>

							<h1 class="cg-page-title entry-title"><?php the_title(); ?></h1>
						</header>
					</div>
				</div>
				<?php
				if ( $economistcommercegurus_share_options == 'yes' ) {
					economistcommercegurus_get_share_options_title();
				}
				?>

			</div>

		</div>
		<?php
		if ( $economistcommercegurus_share_options == 'yes' ) {
			economistcommercegurus_get_share_options_content();
		}
		?>
	<?php } ?>



	<?php if ( $economistcommercegurus_show_page_title == 'Minimal' ) { ?>
		<div class="header-wrapper minimal">
			<div class="container">
				<div class="row">
					<div class="col-xs-12 col-lg-12 col-md-12">
						<header class="entry-header">
							<?php if ( function_exists( 'yoast_breadcrumb' ) ) { ?>
								<?php yoast_breadcrumb( '<p class="sub-title">', '</p>' ); ?>					
							<?php } ?>

							<h1 class="cg-page-title entry-title"><?php the_title(); ?></h1>
						</header>
					</div>
				</div>
				<?php
				if ( $economistcommercegurus_share_options == 'yes' ) {
					economistcommercegurus_get_share_options_title();
				}
				?>

			</div>
		</div>
		<?php
		if ( $economistcommercegurus_share_options == 'yes' ) {
			economistcommercegurus_get_share_options_content();
		}
		?>
	<?php } 
}

function economistcommercegurus_get_page_title() {
	do_action( 'economistcommercegurus_page_title' );
}

function economistcommercegurus_get_share_options_title() {
	?>

	<div class="cg-share btn-group">
		<a class="dropdown-toggle" data-toggle="dropdown" href="#">
			<span id="cg-share-toggle"><?php esc_html_e( 'Share', 'economist' ); ?></span>
		</a>
	</div>

<?php 

	wp_add_inline_script( 
		'economistcommercegurus_commercegurus_js', 'jQuery( document ).ready( function( $ ) { 
			$( function () {
				$( "#cg-share-toggle" ).click( function () {
					$( ".share-wrapper" ).toggleClass( "open" );
					e.preventDefault();
				} );
			} );
		});'
	);
}

function economistcommercegurus_get_share_options_content() {
	?>

	<div class="share-wrapper">
		<div class="container">
			<div class="row">
				<div class="col-lg-12 col-md-12">
					<div class="cg-share-tools">
						<ul>
							<li><a class="social facebook" target="_blank" href="https://facebook.com/sharer.php?u=<?php echo get_permalink(); ?>"><i class="fa fa-facebook"></i></a>
								<span class="social-text"><?php esc_html_e( 'Facebook', 'economist' ); ?></span></li>            
							<li><a class="social twitter" target="_blank" href="https://twitter.com/intent/tweet?url=<?php echo get_permalink(); ?>&amp;text=<?php echo urlencode( get_the_title() ); ?>"><i class="fa fa-twitter"></i></a>
								<span class="social-text"><?php esc_html_e( 'Twitter', 'economist' ); ?></span></li>
							<li><a class="social linkedin" target="_blank" href="https://www.linkedin.com/shareArticle?url=<?php echo get_permalink(); ?>&amp;title=<?php echo urlencode( get_the_title() ); ?>"><i class="fa fa-linkedin"></i></a>
								<span class="social-text"><?php esc_html_e( 'LinkedIn', 'economist' ); ?></span>
							</li>

							<li><a class="social googleplus" target="_blank" href="https://plus.google.com/share?url=<?php echo get_permalink(); ?>"><i class="fa fa-google-plus"></i></a>
								<span class="social-text"><?php esc_html_e( 'Google+', 'economist' ); ?></span>
							</li>                
							<li><a class="social email" target="_blank" href="mailto:?body=<?php echo get_permalink(); ?>"><i class="fa fa-envelope-o"></i></a>
								<span class="social-text"><?php esc_html_e( 'Email', 'economist' ); ?></span>
							</li>
						</ul>
					</div>
				</div>
			</div>
		</div>
	</div>

	<?php
}

function economistcommercegurus_post_share_options() {
	?>

	<aside class="cg-share-post">

		<a class="social facebook" target="_blank" href="https://facebook.com/sharer.php?u=<?php the_permalink(); ?>">
			<i class="fa fa-facebook"></i></a>
		<a class="social twitter" target="_blank" href="https://twitter.com/intent/tweet?url=<?php the_permalink(); ?>&amp;text=<?php echo urlencode( get_the_title() ); ?>">
			<i class="fa fa-twitter"></i></a>
		<a class="social linkedin" target="_blank" href="https://www.linkedin.com/shareArticle?url=<?php the_permalink(); ?>&amp;title=<?php echo urlencode( get_the_title() ); ?>">
			<i class="fa fa-linkedin"></i></a>
		<a class="social googleplus" target="_blank" href="https://plus.google.com/share?url=<?php the_permalink(); ?>">
			<i class="fa fa-google-plus"></i></a>

	</aside><!--/cg-share-post -->

	<?php
}

// Dynamic blog header banner
function economistcommercegurus_blog_header_banner() {
	global $wp_query, $economistcommercegurus_options;
	$post_id = '';

	if ( !is_404() ) {
		if ( isset( $wp_query ) ) {
			if ( $wp_query->have_posts() ) {
				$post_id = $wp_query->post->ID;
			}
		}
	}

	$economistcommercegurus_blog_header_bg				 = '';
	$economistcommercegurus_blog_archive_title_bg_img	 = '';
	$economistcommercegurus_blog_banner_css				 = '';
	$economistcommercegurus_blog_archive_banner_css		 = '';

	if ( is_single() || is_page() ) {
		if ( !empty( $post_id ) ) {
			$economistcommercegurus_blog_header_bg = wp_get_attachment_image_src( get_post_thumbnail_id( $post_id ), 'full' );
		}
		$economistcommercegurus_blog_banner_css .= "
            .cg-hero-bg {
                background-image: url( $economistcommercegurus_blog_header_bg[0] );
            }
        ";
		wp_add_inline_style( 'economistcommercegurus-commercegurus', $economistcommercegurus_blog_banner_css );
	} else if ( is_wc_active() ) {
		if ( ( is_archive() || is_home() ) && (!is_wc_shop() && !is_product_category() && !is_product_tag() ) ) {
			$economistcommercegurus_blog_archive_title_bg_img_array = economistcommercegurus_getoption( 'economistcommercegurus_blog_archive_title_bg_img' );
			if ( $economistcommercegurus_blog_archive_title_bg_img_array ) {
				$economistcommercegurus_blog_archive_title_bg_img = $economistcommercegurus_blog_archive_title_bg_img_array['url'];
			}

			$economistcommercegurus_blog_archive_banner_css .= "
            	body .cg-hero-bg {
                	background-image: url( $economistcommercegurus_blog_archive_title_bg_img );
            	}
        	";
			wp_add_inline_style( 'economistcommercegurus-commercegurus', $economistcommercegurus_blog_archive_banner_css );
		}
	} else if ( ( is_archive() || is_home() ) ) {

		$economistcommercegurus_blog_archive_title_bg_img_array = economistcommercegurus_getoption( 'economistcommercegurus_blog_archive_title_bg_img' );
		if ( $economistcommercegurus_blog_archive_title_bg_img_array ) {
			$economistcommercegurus_blog_archive_title_bg_img = $economistcommercegurus_blog_archive_title_bg_img_array['url'];
		}

		$economistcommercegurus_blog_archive_banner_css .= "
            body .cg-hero-bg {
                background-image: url( $economistcommercegurus_blog_archive_title_bg_img );
            }
        ";
		wp_add_inline_style( 'economistcommercegurus-commercegurus', $economistcommercegurus_blog_archive_banner_css );
	}
}

add_action( 'wp_enqueue_scripts', 'economistcommercegurus_blog_header_banner' );


// WooCommerce Header

add_action( 'economistcommercegurus_woo_title', 'economistcommercegurus_woo_title_callback' );

function economistcommercegurus_woo_title_callback() {
	global $post;
	$economistcommercegurus_show_page_title	 = '';
	$economistcommercegurus_share_options	 = '';
	$economistcommercegurus_share_options	 = economistcommercegurus_getoption( 'economistcommercegurus_share_options' );

	if ( function_exists( 'get_field' ) ) {

		$economistcommercegurus_show_page_title = get_field( 'show_page_title' );
	}
	?>


	<?php if ( $economistcommercegurus_show_page_title !== 'Hide' ) { ?>

		<div class="header-wrapper">
			<div class="cg-hero-bg"></div>
			<div class="overlay"></div> 
			<div class="container">
				<div class="row">
					<div class="col-xs-12 col-lg-12 col-md-12">
						<header class="entry-header">
							<?php if ( function_exists( 'yoast_breadcrumb' ) ) { ?>
								<?php yoast_breadcrumb( '<p class="sub-title">', '</p>' ); ?>		
							<?php } ?>
							<h1 class="cg-page-title entry-title"><?php woocommerce_page_title(); ?></h1>
						</header>
					</div>					
				</div>
				<?php
				if ( $economistcommercegurus_share_options == 'yes' ) {
					economistcommercegurus_get_share_options_title();
				}
				?>
			</div>
		</div>
		<?php
		if ( $economistcommercegurus_share_options == 'yes' ) {
			economistcommercegurus_get_share_options_content();
		}
		?>
	<?php }
	?>

	<?php
}

function economistcommercegurus_get_woo_title() {
	do_action( 'economistcommercegurus_woo_title' );
}

// End Woo Header

add_action( 'economistcommercegurus_blog_page_title', 'economistcommercegurus_blog_page_title_callback' );

function economistcommercegurus_blog_page_title_callback() {

	$economistcommercegurus_blog_page_title	 = '';
	$economistcommercegurus_blog_header_bg	 = '';

	$economistcommercegurus_show_page_title	 = '';
	$economistcommercegurus_cta_heading		 = '';
	$economistcommercegurus_cta_heading		 = economistcommercegurus_getoption( 'economistcommercegurus_cta_heading' );

	$economistcommercegurus_share_options	 = '';
	$economistcommercegurus_share_options	 = economistcommercegurus_getoption( 'economistcommercegurus_share_options' );

	$economistcommercegurus_blog_page_title = economistcommercegurus_getoption( 'economistcommercegurus_blog_page_title' );

	if ( function_exists( 'get_field' ) ) {
		$economistcommercegurus_show_page_title = get_field( 'show_page_title' );
	}
	?>

	<div class="header-wrapper">
		<div class="cg-hero-bg"></div>
		<div class="overlay"></div> 
		<div class="container">
			<div class="row">
				<div class="col-xs-12 col-lg-12 col-md-12">
					<header class="entry-header">
						<?php if ( function_exists( 'yoast_breadcrumb' ) ) { ?>
							<?php yoast_breadcrumb( '<p class="sub-title">', '</p>' ); ?>		
						<?php } ?>
						<h1 class="cg-page-title entry-title"><?php echo wp_kses_post( $economistcommercegurus_blog_page_title ); ?></h1>
					</header>
				</div>					
			</div>
			<?php
			if ( $economistcommercegurus_share_options == 'yes' ) {
				economistcommercegurus_get_share_options_title();
			}
			?>
		</div>
	</div>
	<?php
	if ( $economistcommercegurus_share_options == 'yes' ) {
		economistcommercegurus_get_share_options_content();
	}
	?>
	<?php
}

function economistcommercegurus_get_blog_page_title() {
	do_action( 'economistcommercegurus_blog_page_title' );
}

/* Logo helper */
if ( !function_exists( 'economistcommercegurus_get_logo' ) ) {

	function economistcommercegurus_get_logo( $logo_type ) {
		global $economistcommercegurus_mobile_light_logo;
		$economistcommercegurus_protocol	 = ( is_ssl() ) ? "https:" : "http:";
		$economistcommercegurus_logo		 = '';
		$economistcommercegurus_logo_array	 = '';
		$economistcommercegurus_trans_logo	 = '';
		$trans_site_logo_light				 = '';
		$trans_site_logo_dark				 = '';

		// Logo vars

		if ( $logo_type == 'main' ) {

			// Main logo
			$economistcommercegurus_logo_array = economistcommercegurus_getoption( 'site_logo' );
			if ( $economistcommercegurus_logo_array ) {
				$economistcommercegurus_logo = $economistcommercegurus_logo_array['url'];
			}

			if ( !empty( $economistcommercegurus_logo ) ) {
				$economistcommercegurus_logo = $economistcommercegurus_protocol . str_replace( array( 'http:', 'https:' ), '', $economistcommercegurus_logo );
			}

			// Header styles
			$economistcommercegurus_header_bg_style = '';

			$economistcommercegurus_logo_position	 = '';
			$economistcommercegurus_logo_position	 = economistcommercegurus_getoption( 'economistcommercegurus_logo_position' );

			if ( isset( $_GET['logo_position'] ) ) {
				$economistcommercegurus_logo_position = $_GET['logo_position'];
			}

			if ( ( $economistcommercegurus_logo_position == 'beside' ) || ( $economistcommercegurus_logo_position == 'right' ) ) {

				if ( $economistcommercegurus_logo ) {
					return $economistcommercegurus_logo;
				} else {
					return false;
				}
			} else if ( $economistcommercegurus_logo ) {
				return $economistcommercegurus_logo;
			} else {
				return false;
			}
		} else if ( $logo_type == 'sticky' ) {

			// Sticky logo
			$economistcommercegurus_alt_site_logo_array	 = '';
			$economistcommercegurus_alt_site_logo_array	 = economistcommercegurus_getoption( 'economistcommercegurus_alt_site_logo' );

			if ( $economistcommercegurus_alt_site_logo_array ) {
				$economistcommercegurus_sticky_logo = $economistcommercegurus_alt_site_logo_array['url'];
			}

			if ( !empty( $economistcommercegurus_sticky_logo ) ) {
				$economistcommercegurus_sticky_logo = $economistcommercegurus_protocol . str_replace( array( 'http:', 'https:' ), '', $economistcommercegurus_sticky_logo );
				return $economistcommercegurus_sticky_logo;
			}
		}
	}

}


/* Logo helper */
if ( !function_exists( 'economistcommercegurus_get_bcrumbs' ) ) {

	function economistcommercegurus_get_bcrumbs() {

		$economistcommercegurus_share_options	 = '';
		$economistcommercegurus_share_options	 = economistcommercegurus_getoption( 'economistcommercegurus_share_options' );

		if ( function_exists( 'yoast_breadcrumb' ) ) {
			?>
			<div class="breadcrumbs-wrapper">
				<div class="row">
					<div class="col-lg-9 col-md-9 col-sm-9">
						<?php
						yoast_breadcrumb( '<p class="sub-title">', '</p>' );
						?>
					</div>
					<div class="col-lg-3 col-md-3 col-sm-3">
						<?php
						if ( $economistcommercegurus_share_options == 'yes' ) {
							economistcommercegurus_get_share_options_content();
						}
						?>
					</div>
				</div>
			</div>		
			<?php
		}
	}

}

function economistcommercegurus_safe_html( $input ) {

	$args = array(
		//formatting
		'strong' => array(),
		'em'	 => array(),
		'b'		 => array(),
		'i'		 => array(
			'class' => array()
		),
		'p'		 => array(),
		//links
		'a'		 => array(
			'href' => array()
		)
	);

	return wp_kses( $input, $args );
}
